use serde::Serialize;

use crate::{HResult, User};

use super::OcsJson;

pub async fn get_users(user: User) -> HResult<OcsJson<OcsUsers>> {
    Ok(OcsJson::ok(OcsUsers {
        users: vec!["demo".into()].into(),
    }))
}

#[derive(Serialize)]
pub struct OcsUsers {
    users: Box<[Box<str>]>,
}

pub async fn get_user(user: User) -> HResult<OcsJson<OcsUser>> {
    Ok(OcsJson::ok(OcsUser {
        id: user.id.to_string().into(),
        language: "de".into(),
        enabled: Some(true),
        profile_enabled: "1".into(),
        display_name: user.name.clone(),
        display_name_2: user.name,
        groups: vec!["demo".into()].into(),
        last_login: 0,
        ..Default::default()
    }))
}

#[derive(Serialize, Default)]
pub struct OcsUser {
    additional_mail: Box<[Box<str>]>,
    #[serde(rename = "additional_mailScope")]
    additional_mail_scope: Box<[OcsScope]>,
    address: Box<str>,
    #[serde(rename = "addressScope")]
    address_scope: OcsScope,
    #[serde(rename = "avatarScope")]
    avatar_scope: OcsScope,
    backend: Box<str>,
    #[serde(rename = "backendCapabilities")]
    backend_capabilities: OcsBackendCapabilities,
    biography: Box<str>,
    #[serde(rename = "biographyScope")]
    biography_scope: OcsScope,
    #[serde(rename = "display-name")]
    display_name: Box<str>,
    #[serde(rename = "displayname")]
    display_name_2: Box<str>,
    #[serde(rename = "displaynameScope")]
    display_name_scope: OcsScope,
    email: Option<Box<str>>,
    #[serde(rename = "emailScope")]
    email_scope: OcsScope,
    enabled: Option<bool>,
    fediverse: Box<str>,
    #[serde(rename = "fediverseScope")]
    fediverse_scope: OcsScope,
    groups: Box<[Box<str>]>,
    headline: Box<str>,
    #[serde(rename = "headlineScope")]
    headline_scope: OcsScope,
    id: Box<str>,
    language: Box<str>,
    #[serde(rename = "lastLogin")]
    last_login: i64,
    locale: Box<str>,
    manager: Box<str>,
    notify_email: Option<Box<str>>,
    organisation: Box<str>,
    #[serde(rename = "organisationScope")]
    organisation_scope: OcsScope,
    phone: Box<str>,
    #[serde(rename = "phoneScope")]
    phone_scope: OcsScope,
    profile_enabled: Box<str>,
    #[serde(rename = "profile_enabledScope")]
    profile_enabled_scope: OcsScope,
    quota: OcsQuota,
    role: Box<str>,
    #[serde(rename = "roleScope")]
    role_scope: OcsScope,
    #[serde(rename = "storageLocation")]
    storage_location: Box<str>,
    subadmin: Box<[Box<str>]>,
    twitter: Box<str>,
    #[serde(rename = "twitterScope")]
    twitter_scope: OcsScope,
    website: Box<str>,
    #[serde(rename = "websiteScope")]
    website_scope: OcsScope,
}

#[derive(Serialize, Default)]
#[serde(rename_all = "camelCase")]
pub struct OcsBackendCapabilities {
    set_display_name: bool,
    set_password: bool,
}

#[derive(Serialize, Default)]
pub enum OcsScope {
    #[serde(rename = "v2-private")]
    Private,

    #[default]
    #[serde(rename = "v2-local")]
    Local,

    #[serde(rename = "v2-federated")]
    Federated,

    #[serde(rename = "v2-published")]
    Published,
}

#[derive(Serialize, Default)]
pub struct OcsQuota {
    free: u64,
    quota: i64,
    relative: f32,
    total: u64,
    used: u32,
}
